@@ -6,6 +6,7 @@
 
 #include <algorithm>
 #include <vector>
+#include <limits>
 
 #include <string.h>
 
@@ -226,13 +227,18 @@ public:
 		switch (base) {
 		default:
 		case decimal:
-			if (m_number == -1) {
+			if (m_number == std::numeric_limits<int64_t>::min()) {
+				constexpr int64_t max = (std::numeric_limits<int64_t>::max() - 9) / 10;
 				if (IsNumeric() || IsLeftNumeric()) {
 					m_number = 0;
 					for (unsigned int i = 0; i < m_len; ++i) {
 						if (m_pToken[i] < '0' || m_pToken[i] > '9') {
 							break;
 						}
+						if (m_number > max) {
+							m_number = -1;
+							break;
+						}
 						m_number *= 10;
 						m_number += m_pToken[i] - '0';
 					}
@@ -244,6 +250,10 @@ public:
 						--start;
 					}
 					for (unsigned int i = start; i < m_len; ++i) {
+						if (m_number > max) {
+							m_number = -1;
+							break;
+						}
 						m_number *= 10;
 						m_number += m_pToken[i] - '0';
 					}
@@ -252,8 +262,12 @@ public:
 			return m_number;
 		case hex:
 			{
+				constexpr int64_t max = (std::numeric_limits<int64_t>::max() - 15) / 16;
 				int64_t number = 0;
 				for (unsigned int i = 0; i < m_len; ++i) {
+					if (number > max) {
+						return -1;
+					}
 					const wchar_t& c = m_pToken[i];
 					if (c >= '0' && c <= '9') {
 						number *= 16;
@@ -277,7 +291,7 @@ public:
 	}
 
 protected:
-	int64_t m_number{-1};
+	int64_t m_number{std::numeric_limits<int64_t>::min()};
 
 	wchar_t const* m_pToken{};
 
@@ -289,7 +303,7 @@ protected:
 class CLine final
 {
 public:
-	CLine(std::wstring && line, int trailing_whitespace = -1)
+	CLine(std::wstring && line, size_t trailing_whitespace = std::string::npos)
 		: trailing_whitespace_(trailing_whitespace)
 		, line_(line)
 	{
@@ -315,8 +329,9 @@ public:
 			if (line_[m_parsePos] == ' ' || line_[m_parsePos] == '\t') {
 				m_Tokens.emplace_back(line_.c_str() + start, m_parsePos - start);
 
-				while (m_parsePos < line_.size() && (line_[m_parsePos] == ' ' || line_[m_parsePos] == '\t'))
+				while (m_parsePos < line_.size() && (line_[m_parsePos] == ' ' || line_[m_parsePos] == '\t')) {
 					++m_parsePos;
+				}
 
 				if (m_Tokens.size() > n) {
 					return m_Tokens[n];
@@ -351,10 +366,11 @@ public:
 			}
 			wchar_t const* p = ref.GetToken() + ref.size() + 1;
 
-			auto const newLen = line_.size() - (p - line_.c_str());
-			if (newLen <= 0) {
+			if (static_cast<size_t>(p - line_.c_str()) >= line_.size()) {
 				return CToken();
 			}
+
+			auto newLen = line_.size() - (p - line_.c_str());
 			return CToken(p, newLen);
 		}
 
@@ -368,7 +384,7 @@ public:
 			}
 		}
 
-		if (trailing_whitespace_ == -1) {
+		if (trailing_whitespace_ == std::string::npos) {
 			trailing_whitespace_ = 0;
 			size_t i = line_.size() - 1;
 			while (i < line_.size() && (line_[i] == ' ' || line_[i] == '\t')) {
@@ -380,10 +396,10 @@ public:
 		for (unsigned int i = static_cast<unsigned int>(m_LineEndTokens.size()); i <= n; ++i) {
 			CToken const& refToken = m_Tokens[i];
 			const wchar_t* p = refToken.GetToken();
-			auto const newLen = line_.size() - (p - line_.c_str()) - trailing_whitespace_;
-			if (newLen <= 0) {
+			if ((p - line_.c_str()) + trailing_whitespace_ >= line_.size()) {
 				return CToken();
 			}
+			auto newLen = line_.size() - (p - line_.c_str()) - trailing_whitespace_;
 			m_LineEndTokens.emplace_back(p, newLen);
 		}
 		return m_LineEndTokens[n];
@@ -414,7 +430,7 @@ protected:
 	std::vector<CToken> m_Tokens;
 	std::vector<CToken> m_LineEndTokens;
 	size_t m_parsePos{};
-	int trailing_whitespace_;
+	size_t trailing_whitespace_;
 	std::wstring const line_;
 };
 
@@ -1220,7 +1236,9 @@ bool CDirectoryListingParser::ParseUnixDateTime(CLine & line, int &index, CDiren
 
 		// Some servers use times only for files newer than 6 months
 		if (year <= 0) {
-			assert(month != -1 && day != -1);
+			if (month == -1 || day == -1) {
+				return false;
+			}
 			tm const t = fz::datetime::now().get_tm(fz::datetime::utc);
 			year = t.tm_year + 1900;
 			int const currentDayOfYear = t.tm_mday + 31 * t.tm_mon;
@@ -1449,9 +1467,9 @@ bool CDirectoryListingParser::ParseShortDate(CToken &token, CDirentry &entry, bo
 		gotYear = true;
 	}
 
-	assert(gotYear);
-	assert(gotMonth);
-	assert(gotDay);
+	if (!gotYear || !gotMonth || !gotDay) {
+		return false;
+	}
 
 	if (!entry.time.set(fz::datetime::utc, year, month, day)) {
 		return false;
