@@ -3,9 +3,10 @@
 
 #include "directorycache.h"
 #include "logging_private.h"
+#include "oplock_manager.h"
 #include "pathcache.h"
 #include "ratelimiter.h"
-#include "socket.h"
+#include "tls_system_trust_store.h"
 
 #include <libfilezilla/event_loop.hpp>
 #include <libfilezilla/thread_pool.hpp>
@@ -49,8 +50,11 @@ public:
 	Impl(COptionsBase& options)
 		: limiter_(loop_, options)
 		, optionChangeHandler_(options, loop_)
+		, tlsSystemTrustStore_(pool_)
 	{
 		CLogging::UpdateLogLevel(options);
+
+		directory_cache_.SetTtl(fz::duration::from_seconds(options.GetOptionVal(OPTION_CACHE_TTL)));
 	}
 
 	~Impl()
@@ -59,15 +63,18 @@ public:
 	}
 
 	fz::thread_pool pool_;
-	fz::event_loop loop_;
+	fz::event_loop loop_{pool_};
 	CRateLimiter limiter_;
 	CDirectoryCache directory_cache_;
 	CPathCache path_cache_;
 	CLoggingOptionsChanged optionChangeHandler_;
+	OpLockManager opLockManager_;
+	TlsSystemTrustStore tlsSystemTrustStore_;
 };
 
-CFileZillaEngineContext::CFileZillaEngineContext(COptionsBase & options)
+CFileZillaEngineContext::CFileZillaEngineContext(COptionsBase & options, CustomEncodingConverterBase const& customEncodingConverter)
 : options_(options)
+, customEncodingConverter_(customEncodingConverter)
 , impl_(new Impl(options))
 {
 }
@@ -76,11 +83,6 @@ CFileZillaEngineContext::~CFileZillaEngineContext()
 {
 }
 
-COptionsBase& CFileZillaEngineContext::GetOptions()
-{
-	return options_;
-}
-
 fz::thread_pool& CFileZillaEngineContext::GetThreadPool()
 {
 	return impl_->pool_;
@@ -105,3 +107,13 @@ CPathCache& CFileZillaEngineContext::GetPathCache()
 {
 	return impl_->path_cache_;
 }
+
+OpLockManager& CFileZillaEngineContext::GetOpLockManager()
+{
+	return impl_->opLockManager_;
+}
+
+TlsSystemTrustStore& CFileZillaEngineContext::GetTlsSystemTrustStore()
+{
+	return impl_->tlsSystemTrustStore_;
+}
