@@ -34,15 +34,6 @@
 CFtpControlSocket::CFtpControlSocket(CFileZillaEnginePrivate & engine)
 	: CRealControlSocket(engine)
 {
-	// Enable TCP_NODELAY, speeds things up a bit.
-	socket_->set_flags(fz::socket::flag_nodelay | fz::socket::flag_keepalive);
-
-	// Enable SO_KEEPALIVE, lots of clueless users have broken routers and
-	// firewalls which terminate the control connection on long transfers.
-	int v = engine_.GetOptions().GetOptionVal(OPTION_TCP_KEEPALIVE_INTERVAL);
-	if (v >= 1 && v < 10000) {
-		socket_->set_keepalive_interval(fz::duration::from_minutes(v));
-	}
 }
 
 CFtpControlSocket::~CFtpControlSocket()
@@ -58,7 +49,7 @@ void CFtpControlSocket::OnReceive()
 
 	for (;;) {
 		int error;
-		int read = m_pBackend->Read(m_receiveBuffer + m_bufferLen, RECVBUFFERSIZE - m_bufferLen, error);
+		int read = active_layer_->read(m_receiveBuffer + m_bufferLen, RECVBUFFERSIZE - m_bufferLen, error);
 
 		if (read < 0) {
 			if (error != EAGAIN) {
@@ -182,16 +173,13 @@ void CFtpControlSocket::OnConnect()
 	SetAlive();
 
 	if (currentServer_.GetProtocol() == FTPS) {
-		if (!m_pTlsSocket) {
+		if (!tls_layer_) {
 			LogMessage(MessageType::Status, _("Connection established, initializing TLS..."));
 
-			assert(!m_pTlsSocket);
-			delete m_pBackend;
-			m_pTlsSocket = new CTlsSocket(this, *socket_, this);
-			m_pBackend = m_pTlsSocket;
+			tls_layer_ = std::make_unique<CTlsSocket>(this, *active_layer_, this);
+			active_layer_ = tls_layer_.get();
 
-			int res = m_pTlsSocket->Handshake();
-			if (res == FZ_REPLY_ERROR) {
+			if (!tls_layer_->client_handshake()) {
 				DoClose();
 			}
 
@@ -201,7 +189,7 @@ void CFtpControlSocket::OnConnect()
 			LogMessage(MessageType::Status, _("TLS connection established, waiting for welcome message..."));
 		}
 	}
-	else if ((currentServer_.GetProtocol() == FTPES || currentServer_.GetProtocol() == FTP) && m_pTlsSocket) {
+	else if ((currentServer_.GetProtocol() == FTPES || currentServer_.GetProtocol() == FTP) && tls_layer_) {
 		LogMessage(MessageType::Status, _("TLS connection established."));
 		SendNextCommand();
 		return;
@@ -530,13 +518,13 @@ bool CFtpControlSocket::SetAsyncRequestReply(CAsyncRequestNotification *pNotific
 		break;
 	case reqId_certificate:
 		{
-			if (!m_pTlsSocket || m_pTlsSocket->GetState() != CTlsSocket::TlsState::verifycert) {
+			if (!tls_layer_ || tls_layer_->get_state() != fz::socket_state::connecting) {
 				LogMessage(MessageType::Debug_Info, L"No or invalid operation in progress, ignoring request reply %d", pNotification->GetRequestID());
 				return false;
 			}
 
 			CCertificateNotification* pCertificateNotification = static_cast<CCertificateNotification *>(pNotification);
-			m_pTlsSocket->TrustCurrentCert(pCertificateNotification->m_trusted);
+			tls_layer_->TrustCurrentCert(pCertificateNotification->m_trusted);
 
 			if (!pCertificateNotification->m_trusted) {
 				DoClose(FZ_REPLY_CRITICALERROR);
@@ -723,7 +711,7 @@ void CFtpControlSocket::Transfer(std::wstring const& cmd, CFtpTransferOpData* ol
 	pData->pOldData = oldData;
 	pData->pOldData->transferEndReason = TransferEndReason::successful;
 
-	if (m_pProxyBackend) {
+	if (proxy_layer_) {
 		// Only passive suported
 		// Theoretically could use reverse proxy ability in SOCKS5, but
 		// it is too fragile to set up with all those broken routers and
@@ -853,3 +841,9 @@ void CFtpControlSocket::operator()(fz::event_base const& ev)
 
 	CRealControlSocket::operator()(ev);
 }
+
+void CFtpControlSocket::ResetSocket()
+{
+	tls_layer_.reset();
+	CRealControlSocket::ResetSocket();
+}
