@@ -6,7 +6,7 @@
 #include "../tlssocket.h"
 
 CFtpLogonOpData::CFtpLogonOpData(CFtpControlSocket& controlSocket, Credentials const& credentials)
-    : COpData(Command::connect, L"CFtpLogonOpData")
+	: COpData(Command::connect, L"CFtpLogonOpData")
 	, CFtpOpData(controlSocket)
 	, credentials_(credentials)
 {
@@ -41,10 +41,10 @@ int CFtpLogonOpData::Send()
 	switch (opState)
 	{
 	case LOGON_CONNECT:
-	    {
+		{
 			// Do not use FTP proxy if generic proxy is set
 			int const generic_proxy_type = engine_.GetOptions().GetOptionVal(OPTION_PROXY_TYPE);
-			if ((generic_proxy_type <= CProxySocket::unknown || generic_proxy_type >= CProxySocket::proxytype_count) && !currentServer_.GetBypassProxy()) {
+			if ((generic_proxy_type <= static_cast<int>(ProxyType::NONE) || generic_proxy_type >= static_cast<int>(ProxyType::count)) && !currentServer_.GetBypassProxy()) {
 				ftp_proxy_type_ = engine_.GetOptions().GetOptionVal(OPTION_FTP_PROXY_TYPE);
 			}
 		
@@ -99,8 +99,20 @@ int CFtpLogonOpData::Send()
 			}
 
 			opState = LOGON_WELCOME;
-			return controlSocket_.DoConnect(host_, port_);
-	    }
+			int ret = controlSocket_.DoConnect(host_, port_);
+			if (ret == FZ_REPLY_WOULDBLOCK) {
+				// Enable TCP_NODELAY, speeds things up a bit.
+				controlSocket_.socket_->set_flags(fz::socket::flag_nodelay | fz::socket::flag_keepalive);
+
+				// Enable SO_KEEPALIVE, lots of clueless users have broken routers and
+				// firewalls which terminate the control connection on long transfers.
+				int v = engine_.GetOptions().GetOptionVal(OPTION_TCP_KEEPALIVE_INTERVAL);
+				if (v >= 1 && v < 10000) {
+					controlSocket_.socket_->set_keepalive_interval(fz::duration::from_minutes(v));
+				}
+			}
+			return ret;
+		}
 	case LOGON_AUTH_WAIT:
 		LogMessage(MessageType::Debug_Info, L"LogonSend() called during LOGON_AUTH_WAIT, ignoring");
 		return FZ_REPLY_WOULDBLOCK;
@@ -111,8 +123,8 @@ int CFtpLogonOpData::Send()
 	case LOGON_SYST:
 		return controlSocket_.SendCommand(L"SYST");
 	case LOGON_LOGON:
-	    {
-		    t_loginCommand cmd = loginSequence.front();
+		{
+			t_loginCommand cmd = loginSequence.front();
 			switch (cmd.type)
 			{
 			case loginCommandType::user:
@@ -168,7 +180,7 @@ int CFtpLogonOpData::Send()
 			default:
 				return FZ_REPLY_INTERNALERROR;
 			}
-	    }
+		}
 		break;
 	case LOGON_FEAT:
 		return controlSocket_.SendCommand(L"FEAT");
@@ -197,11 +209,11 @@ int CFtpLogonOpData::Send()
 		}
 		return controlSocket_.SendCommand(currentServer_.GetPostLoginCommands()[customCommandIndex]);
 	case LOGON_OPTSMLST:
-	    {
-		    std::wstring args;
+		{
+			std::wstring args;
 			CServerCapabilities::GetCapability(currentServer_, opst_mlst_command, &args);
 			return controlSocket_.SendCommand(L"OPTS MLST " + args);
-	    }
+		}
 		break;
 	default:
 		LogMessage(MessageType::Debug_Warning, L"unknown op state: %d", opState);
@@ -247,15 +259,11 @@ int CFtpLogonOpData::ParseResponse()
 
 			LogMessage(MessageType::Status, _("Initializing TLS..."));
 
-			assert(!controlSocket_.m_pTlsSocket);
-			delete controlSocket_.m_pBackend;
+			controlSocket_.tls_layer_ = std::make_unique<CTlsSocket>(&controlSocket_, *controlSocket_.active_layer_, &controlSocket_);
+			controlSocket_.active_layer_ = controlSocket_.tls_layer_.get();
 
-			controlSocket_.m_pTlsSocket = new CTlsSocket(&controlSocket_, *controlSocket_.socket_, &controlSocket_);
-			controlSocket_.m_pBackend = controlSocket_.m_pTlsSocket;
-
-			int res = controlSocket_.m_pTlsSocket->Handshake();
-			if (res & FZ_REPLY_ERROR) {
-				return res | FZ_REPLY_DISCONNECTED;
+			if (!controlSocket_.tls_layer_->client_handshake()) {
+				return FZ_REPLY_ERROR | FZ_REPLY_DISCONNECTED;
 			}
 
 			neededCommands[LOGON_AUTH_SSL] = 0;
