@@ -1,5 +1,5 @@
-#ifndef __SERVERCAPABILITIES_H__
-#define __SERVERCAPABILITIES_H__
+#ifndef FILEZILLA_ENGINE_SERVERCAPABILITIES_HEADER
+#define FILEZILLA_ENGINE_SERVERCAPABILITIES_HEADER
 
 enum capabilities
 {
@@ -29,9 +29,6 @@ enum capabilityNames
 	rest_stream, // supports REST+STOR in addition to APPE
 	epsv_command,
 
-	// FTPS and HTTPS
-	tls_resume, // Does the server support resuming of TLS sessions?
-
 	// Server timezone offset. If using FTP, LIST details are unspecified and
 	// can return different times than the UTC based times using the MLST or
 	// MDTM commands.
@@ -47,16 +44,16 @@ enum capabilityNames
 class CCapabilities final
 {
 public:
-	capabilities GetCapability(capabilityNames name, wxString* pOption = 0) const;
+	capabilities GetCapability(capabilityNames name, std::wstring* pOption = nullptr) const;
 	capabilities GetCapability(capabilityNames name, int* pOption) const;
-	void SetCapability(capabilityNames name, capabilities cap, const wxString& option = _T(""));
+	void SetCapability(capabilityNames name, capabilities cap, std::wstring const& option = std::wstring());
 	void SetCapability(capabilityNames name, capabilities cap, int option);
 
 protected:
 	struct t_cap
 	{
 		capabilities cap;
-		wxString option;
+		std::wstring option;
 		int number;
 	};
 	std::map<capabilityNames, t_cap> m_capabilityMap;
@@ -66,14 +63,14 @@ class CServerCapabilities final
 {
 public:
 	// If return value isn't 'yes', pOptions remains unchanged
-	static capabilities GetCapability(const CServer& server, capabilityNames name, wxString* pOption = 0);
+	static capabilities GetCapability(const CServer& server, capabilityNames name, std::wstring* pOption = nullptr);
 	static capabilities GetCapability(const CServer& server, capabilityNames name, int* option);
 
-	static void SetCapability(const CServer& server, capabilityNames name, capabilities cap, const wxString& option = _T(""));
+	static void SetCapability(const CServer& server, capabilityNames name, capabilities cap, std::wstring const& option = std::wstring());
 	static void SetCapability(const CServer& server, capabilityNames name, capabilities cap, int option);
 
 protected:
 	static std::map<CServer, CCapabilities> m_serverMap;
 };
 
-#endif //__SERVERCAPABILITIES_H__
+#endif
