@@ -4,11 +4,12 @@
 #include "tlssocket.h"
 #include "tlssocket_impl.h"
 
-CTlsSocket::CTlsSocket(fz::event_handler* pEvtHandler, fz::socket& pSocket, CControlSocket* pOwner)
+CTlsSocket::CTlsSocket(fz::event_handler* pEvtHandler, fz::socket_interface & next_layer, CControlSocket* pOwner)
 	: event_handler(pOwner->event_loop_)
-	, CBackend(pEvtHandler)
+	, SocketLayer(pEvtHandler, next_layer, false)
 {
-	impl_ = std::make_unique<CTlsSocketImpl>(*this, pSocket, pOwner);
+	impl_ = std::make_unique<CTlsSocketImpl>(*this, pOwner);
+	next_layer.set_event_handler(this);
 }
 
 CTlsSocket::~CTlsSocket()
@@ -16,29 +17,24 @@ CTlsSocket::~CTlsSocket()
 	remove_handler();
 }
 
-int CTlsSocket::Handshake(CTlsSocket const* pPrimarySocket, bool try_resume)
+bool CTlsSocket::client_handshake(std::vector<uint8_t> const& session_to_resume, std::vector<uint8_t> const& required_certificate)
 {
-	return impl_->Handshake(pPrimarySocket ? pPrimarySocket->impl_.get() : nullptr, try_resume);
+	return impl_->client_handshake(session_to_resume, required_certificate);
 }
 
-int CTlsSocket::Read(void *buffer, unsigned int size, int& error)
+int CTlsSocket::read(void *buffer, unsigned int size, int& error)
 {
-	return impl_->Read(buffer, size, error);
+	return impl_->read(buffer, size, error);
 }
 
-int CTlsSocket::Peek(void *buffer, unsigned int size, int& error)
+int CTlsSocket::write(void const* buffer, unsigned int size, int& error)
 {
-	return impl_->Peek(buffer, size, error);
+	return impl_->write(buffer, size, error);
 }
 
-int CTlsSocket::Write(const void *buffer, unsigned int size, int& error)
+int CTlsSocket::shutdown()
 {
-	return impl_->Write(buffer, size, error);
-}
-
-int CTlsSocket::Shutdown(bool silenceReadErrors)
-{
-	return impl_->Shutdown(silenceReadErrors);
+	return impl_->shutdown();
 }
 
 void CTlsSocket::TrustCurrentCert(bool trusted)
@@ -46,9 +42,9 @@ void CTlsSocket::TrustCurrentCert(bool trusted)
 	return impl_->TrustCurrentCert(trusted);
 }
 
-CTlsSocket::TlsState CTlsSocket::GetState() const
+fz::socket_state CTlsSocket::get_state() const
 {
-	return impl_->GetState();
+	return impl_->get_state();
 }
 
 std::wstring CTlsSocket::GetProtocolName()
@@ -96,12 +92,22 @@ void CTlsSocket::operator()(fz::event_base const& ev)
 	return impl_->operator()(ev);
 }
 
-void CTlsSocket::OnRateAvailable(CRateLimiter::rate_direction direction)
-{
-	return impl_->OnRateAvailable(direction);
-}
-
 std::wstring CTlsSocket::GetGnutlsVersion()
 {
 	return CTlsSocketImpl::GetGnutlsVersion();
 }
+
+std::vector<uint8_t> CTlsSocket::get_session_parameters() const
+{
+	return impl_->get_session_parameters();
+}
+
+std::vector<uint8_t> CTlsSocket::get_raw_certificate() const
+{
+	return impl_->get_raw_certificate();
+}
+
+int CTlsSocket::connect(fz::native_string const& host, unsigned int port, fz::address_type family)
+{
+	return impl_->connect(host, port, family);
+}
