@@ -32,7 +32,7 @@ char const ciphers[] = "@SYSTEM";
 #if TLSDEBUG
 // This is quite ugly
 CControlSocket* pLoggingControlSocket;
-void log_func(int level, const char* msg)
+void log_func(int level, char const* msg)
 {
 	if (!msg || !pLoggingControlSocket) {
 		return;
@@ -109,15 +109,26 @@ struct datum_holder final : gnutls_datum_t
 {
 	datum_holder() {
 		data = nullptr;
+		size = 0;
 	}
 
 	~datum_holder() {
 		gnutls_free(data);
 	}
 
+	void clear()
+	{
+		gnutls_free(data);
+		data = nullptr;
+		size = 0;
+	}
+
 	datum_holder(datum_holder const&) = delete;
 	datum_holder& operator=(datum_holder const&) = delete;
 
+	std::string to_string() const {
+		return data ? std::string(data, data + size) : std::string();
+	}
 };
 
 void clone_cert(gnutls_x509_crt_t in, gnutls_x509_crt_t &out)
@@ -138,11 +149,9 @@ void clone_cert(gnutls_x509_crt_t in, gnutls_x509_crt_t &out)
 }
 }
 
-CTlsSocketImpl::CTlsSocketImpl(CTlsSocket& tlsSocket, fz::socket& socket, CControlSocket* pOwner)
+CTlsSocketImpl::CTlsSocketImpl(CTlsSocket& tlsSocket, CControlSocket* pOwner)
 	: tlsSocket_(tlsSocket)
 	, m_pOwner(pOwner)
-	, m_socket(socket)
-	, socketBackend_(std::make_unique<CSocketBackend>(static_cast<fz::event_handler*>(&tlsSocket_), m_socket, m_pOwner->GetEngine().GetRateLimiter()))
 {
 }
 
@@ -262,12 +271,7 @@ void CTlsSocketImpl::Uninit()
 		gnutls_global_deinit();
 	}
 
-	m_tlsState = CTlsSocket::TlsState::noconn;
-
-	peekBuffer_.clear();
-
-	delete [] m_implicitTrustedCert.data;
-	m_implicitTrustedCert.data = nullptr;
+	state_ = fz::socket_state::failed;
 
 #if TLSDEBUG
 	if (pLoggingControlSocket == m_pOwner) {
@@ -288,7 +292,7 @@ void CTlsSocketImpl::UninitSession()
 
 void CTlsSocketImpl::LogError(int code, std::wstring const& function, MessageType logLevel)
 {
-	if (logLevel < MessageType::Debug_Warning && m_tlsState == CTlsSocket::TlsState::closed && shutdown_silence_read_errors_) {
+	if (logLevel < MessageType::Debug_Warning && state_ >= fz::socket_state::shut_down && shutdown_silence_read_errors_) {
 		logLevel = MessageType::Debug_Warning;
 	}
 
@@ -312,7 +316,7 @@ void CTlsSocketImpl::LogError(int code, std::wstring const& function, MessageTyp
 		}
 	}
 	else {
-		const char* error = gnutls_strerror(code);
+		char const* error = gnutls_strerror(code);
 		if (error) {
 			if (function.empty()) {
 				m_pOwner->LogMessage(logLevel, _("GnuTLS error %d: %s"), code, error);
@@ -335,7 +339,7 @@ void CTlsSocketImpl::LogError(int code, std::wstring const& function, MessageTyp
 void CTlsSocketImpl::PrintAlert(MessageType logLevel)
 {
 	gnutls_alert_description_t last_alert = gnutls_alert_get(m_session);
-	const char* alert = gnutls_alert_get_name(last_alert);
+	char const* alert = gnutls_alert_get_name(last_alert);
 	if (alert) {
 		m_pOwner->LogMessage(logLevel, _("Received TLS alert from the server: %s (%d)"), alert, last_alert);
 	}
@@ -354,7 +358,7 @@ ssize_t CTlsSocketImpl::PullFunction(gnutls_transport_ptr_t ptr, void* data, siz
 	return ((CTlsSocketImpl*)ptr)->PullFunction(data, len);
 }
 
-ssize_t CTlsSocketImpl::PushFunction(const void* data, size_t len)
+ssize_t CTlsSocketImpl::PushFunction(void const* data, size_t len)
 {
 #if TLSDEBUG
 	m_pOwner->LogMessage(MessageType::Debug_Debug, L"CTlsSocketImpl::PushFunction(%d)", len);
@@ -365,7 +369,7 @@ ssize_t CTlsSocketImpl::PushFunction(const void* data, size_t len)
 	}
 
 	int error;
-	int written = socketBackend_->Write(data, static_cast<unsigned int>(len), error);
+	int written = tlsSocket_.next_layer_.write(data, static_cast<unsigned int>(len), error);
 
 	if (written < 0) {
 		m_canWriteToSocket = false;
@@ -398,7 +402,7 @@ ssize_t CTlsSocketImpl::PullFunction(void* data, size_t len)
 	}
 
 	int error;
-	int read = socketBackend_->Read(data, static_cast<unsigned int>(len), error);
+	int read = tlsSocket_.next_layer_.read(data, static_cast<unsigned int>(len), error);
 	if (read < 0) {
 		m_canReadFromSocket = false;
 		if (error != EAGAIN) {
@@ -424,18 +428,31 @@ ssize_t CTlsSocketImpl::PullFunction(void* data, size_t len)
 
 void CTlsSocketImpl::operator()(fz::event_base const& ev)
 {
-	fz::dispatch<fz::socket_event>(ev, this, &CTlsSocketImpl::OnSocketEvent);
+	fz::dispatch<fz::socket_event, fz::hostaddress_event>(ev, this
+		, &CTlsSocketImpl::OnSocketEvent
+		, &CTlsSocketImpl::forward_hostaddress_event);
 }
 
-void CTlsSocketImpl::OnSocketEvent(fz::socket_event_source*, fz::socket_event_flag t, int error)
+void CTlsSocketImpl::forward_hostaddress_event(fz::socket_event_source* source, std::string const& address)
+{
+	tlsSocket_.forward_hostaddress_event(source, address);
+}
+
+void CTlsSocketImpl::OnSocketEvent(fz::socket_event_source* s, fz::socket_event_flag t, int error)
 {
 	if (!m_session) {
 		return;
 	}
 
+	if (t == fz::socket_event_flag::connection_next) {
+		tlsSocket_.forward_socket_event(s, t, error);
+		return;
+	}
+
 	if (error) {
 		m_socket_error = error;
-		Failure(0, true);
+		Uninit();
+		tlsSocket_.forward_socket_event(s, t, error);
 		return;
 	}
 
@@ -447,6 +464,10 @@ void CTlsSocketImpl::OnSocketEvent(fz::socket_event_source*, fz::socket_event_fl
 	case fz::socket_event_flag::write:
 		OnSend();
 		break;
+	case fz::socket_event_flag::connection:
+		if (hostname_.empty()) {
+			set_hostname(tlsSocket_.next_layer_.peer_host());
+		}
 	default:
 		break;
 	}
@@ -462,17 +483,11 @@ void CTlsSocketImpl::OnRead()
 		return;
 	}
 
-	if (m_tlsState == CTlsSocket::TlsState::handshake) {
-		int const direction = gnutls_record_get_direction(m_session);
-		if (direction) {
-			m_pOwner->LogMessage(MessageType::Debug_Debug, L"CTlsSocketImpl::Postponing read");
-			return;
-		}
+	if (state_ == fz::socket_state::connecting) {
 		ContinueHandshake();
 	}
-	else if (m_tlsState >= CTlsSocket::TlsState::conn) {
-		if (m_lastReadFailed) {
-			m_lastReadFailed = false;
+	else if (state_ == fz::socket_state::connected || state_ == fz::socket_state::shutting_down || state_ == fz::socket_state::shut_down) {
+		if (tlsSocket_.m_pEvtHandler) {
 			tlsSocket_.m_pEvtHandler->send_event<fz::socket_event>(&tlsSocket_, fz::socket_event_flag::read, 0);
 		}
 	}
@@ -488,17 +503,18 @@ void CTlsSocketImpl::OnSend()
 		return;
 	}
 
-	if (m_tlsState == CTlsSocket::TlsState::handshake) {
-		int const direction = gnutls_record_get_direction(m_session);
-		if (!direction) {
-			return;
-		}
+	if (state_ == fz::socket_state::connecting) {
 		ContinueHandshake();
 	}
-	else if (m_tlsState == CTlsSocket::TlsState::closing) {
-		ContinueShutdown();
+	else if (state_ == fz::socket_state::shutting_down) {
+		int res = ContinueShutdown();
+		if (res != EAGAIN) {
+			if (tlsSocket_.m_pEvtHandler) {
+				tlsSocket_.m_pEvtHandler->send_event<fz::socket_event>(&tlsSocket_, fz::socket_event_flag::write, res);
+			}
+		}
 	}
-	else if (m_tlsState == CTlsSocket::TlsState::conn) {
+	else if (state_ == fz::socket_state::connected) {
 		ContinueWrite();
 	}
 }
@@ -522,33 +538,10 @@ void CTlsSocketImpl::ContinueWrite()
 
 		m_writeSkip += static_cast<int>(res);
 		m_lastWriteFailed = false;
-		tlsSocket_.m_pEvtHandler->send_event<fz::socket_event>(&tlsSocket_, fz::socket_event_flag::write, 0);
-	}
-}
-
-bool CTlsSocketImpl::CopySessionData(const CTlsSocketImpl* pPrimarySocket)
-{
-	datum_holder d;
-	int res = gnutls_session_get_data2(pPrimarySocket->m_session, &d);
-	if (res) {
-		m_pOwner->LogMessage(MessageType::Debug_Warning, L"gnutls_session_get_data2 on primary socket failed: %d", res);
-		return true;
-	}
-
-	// Set session data
-	res = gnutls_session_set_data(m_session, d.data, d.size );
-	if (res) {
-		m_pOwner->LogMessage(MessageType::Debug_Info, L"gnutls_session_set_data failed: %d. Going to reinitialize session.", res);
-		UninitSession();
-		if (!InitSession()) {
-			return false;
+		if (tlsSocket_.m_pEvtHandler) {
+			tlsSocket_.m_pEvtHandler->send_event<fz::socket_event>(&tlsSocket_, fz::socket_event_flag::write, 0);
 		}
 	}
-	else {
-		m_pOwner->LogMessage(MessageType::Debug_Info, L"Trying to resume existing TLS session.");
-	}
-
-	return true;
 }
 
 bool CTlsSocketImpl::ResumedSession() const
@@ -556,65 +549,34 @@ bool CTlsSocketImpl::ResumedSession() const
 	return gnutls_session_is_resumed(m_session) != 0;
 }
 
-int CTlsSocketImpl::Handshake(const CTlsSocketImpl* pPrimarySocket, bool try_resume)
+bool CTlsSocketImpl::client_handshake(std::vector<uint8_t> const& session_to_resume, std::vector<uint8_t> const& required_certificate)
 {
-	m_pOwner->LogMessage(MessageType::Debug_Verbose, L"CTlsSocketImpl::Handshake()");
+	m_pOwner->LogMessage(MessageType::Debug_Verbose, L"CTlsSocketImpl::client_handshake()");
 
-	if (m_tlsState != CTlsSocket::TlsState::noconn) {
-		m_pOwner->LogMessage(MessageType::Debug_Warning, L"Called CTlsSocketImpl::Handshake on a socket that isn't idle");
-		return FZ_REPLY_ERROR;
+	if (state_ != fz::socket_state::none) {
+		m_pOwner->LogMessage(MessageType::Debug_Warning, L"Called CTlsSocketImpl::client_handshake on a socket that isn't idle");
+		return false;
 	}
 
 	if (!Init()) {
-		return FZ_REPLY_ERROR;
+		return false;
 	}
 
-	m_tlsState = CTlsSocket::TlsState::handshake;
+	state_ = fz::socket_state::connecting;
 
-	if (pPrimarySocket) {
-		if (!pPrimarySocket->m_session) {
-			m_pOwner->LogMessage(MessageType::Debug_Warning, L"Primary socket has no session");
-			Uninit();
-			return FZ_REPLY_ERROR;
-		}
+	required_certificate_ = required_certificate;
 
-		// Implicitly trust certificate of primary socket
-		unsigned int cert_list_size;
-		const gnutls_datum_t* const cert_list = gnutls_certificate_get_peers(pPrimarySocket->m_session, &cert_list_size);
-		if (cert_list && cert_list_size) {
-			delete [] m_implicitTrustedCert.data;
-			m_implicitTrustedCert.data = new unsigned char[cert_list[0].size];
-			memcpy(m_implicitTrustedCert.data, cert_list[0].data, cert_list[0].size);
-			m_implicitTrustedCert.size = cert_list[0].size;
-		}
-
-		if (try_resume) {
-			if (!CopySessionData(pPrimarySocket)) {
-				return FZ_REPLY_ERROR;
+	if (!session_to_resume.empty()) {
+		int res = gnutls_session_set_data(m_session, session_to_resume.data(), session_to_resume.size());
+		if (res) {
+			m_pOwner->LogMessage(MessageType::Debug_Info, L"gnutls_session_set_data failed: %d. Going to reinitialize session.", res);
+			UninitSession();
+			if (!InitSession()) {
+				return false;
 			}
 		}
-
-		hostname_ = pPrimarySocket->m_socket.peer_host();
-	}
-	else {
-		hostname_ = m_socket.peer_host();
-	}
-
-	int port, tmp;
-	port = m_socket.remote_port(tmp);
-	if (port <= 0) {
-		Uninit();
-		return FZ_REPLY_ERROR;
-	}
-	port_ = port;
-
-	if (!hostname_.empty() && fz::get_address_type(hostname_) == fz::address_type::unknown) {
-		auto const utf8 = fz::to_utf8(hostname_);
-		if (!utf8.empty()) {
-			int res = gnutls_server_name_set(m_session, GNUTLS_NAME_DNS, utf8.c_str(), utf8.size());
-			if (res) {
-				LogError(res, L"gnutls_server_name_set", MessageType::Debug_Warning);
-			}
+		else {
+			m_pOwner->LogMessage(MessageType::Debug_Info, L"Trying to resume existing TLS session.");
 		}
 	}
 
@@ -622,14 +584,19 @@ int CTlsSocketImpl::Handshake(const CTlsSocketImpl* pPrimarySocket, bool try_res
 		gnutls_handshake_set_hook_function(m_session, GNUTLS_HANDSHAKE_ANY, GNUTLS_HOOK_BOTH, &handshake_hook_func);
 	}
 
-	return ContinueHandshake();
+	if (tlsSocket_.next_layer_.get_state() != fz::socket_state::connected) {
+		return true;
+	}
+
+	set_hostname(tlsSocket_.next_layer_.peer_host());
+	return ContinueHandshake() == EAGAIN;
 }
 
 int CTlsSocketImpl::ContinueHandshake()
 {
 	m_pOwner->LogMessage(MessageType::Debug_Verbose, L"CTlsSocketImpl::ContinueHandshake()");
 	assert(m_session);
-	assert(m_tlsState == CTlsSocket::TlsState::handshake);
+	assert(state_ == fz::socket_state::connecting);
 
 	int res = gnutls_handshake(m_session);
 	while (res == GNUTLS_E_AGAIN || res == GNUTLS_E_INTERRUPTED) {
@@ -640,6 +607,7 @@ int CTlsSocketImpl::ContinueHandshake()
 	}
 	if (!res) {
 		m_pOwner->LogMessage(MessageType::Debug_Info, L"TLS Handshake successful");
+		handshake_successful_ = true;
 
 		if (ResumedSession()) {
 			m_pOwner->LogMessage(MessageType::Debug_Info, L"TLS Session resumed");
@@ -652,51 +620,27 @@ int CTlsSocketImpl::ContinueHandshake()
 
 		m_pOwner->LogMessage(MessageType::Debug_Info, L"Protocol: %s, Key exchange: %s, Cipher: %s, MAC: %s", protocol, keyExchange, cipherName, macName);
 
-		res = VerifyCertificate();
-		if (res != FZ_REPLY_OK) {
-			return res;
-		}
-
-		if (shutdown_requested_) {
-			int error = Shutdown(shutdown_silence_read_errors_);
-			if (!error || error != EAGAIN) {
-				tlsSocket_.m_pEvtHandler->send_event<fz::socket_event>(&tlsSocket_, fz::socket_event_flag::read, error);
-			}
-		}
-
-		return FZ_REPLY_OK;
+		return VerifyCertificate();
 	}
 	else if (res == GNUTLS_E_AGAIN || res == GNUTLS_E_INTERRUPTED) {
-		return FZ_REPLY_WOULDBLOCK;
+		return EAGAIN;
 	}
 
 	Failure(res, true);
 
-	return FZ_REPLY_ERROR;
+	return m_socket_error ? m_socket_error : ECONNABORTED;
 }
 
-int CTlsSocketImpl::Read(void *buffer, unsigned int len, int& error)
+int CTlsSocketImpl::read(void *buffer, unsigned int len, int& error)
 {
-	if (m_tlsState == CTlsSocket::TlsState::handshake || m_tlsState == CTlsSocket::TlsState::verifycert) {
+	if (state_ == fz::socket_state::connecting) {
 		error = EAGAIN;
 		return -1;
 	}
-	else if (m_tlsState == CTlsSocket::TlsState::noconn) {
+	else if (state_ != fz::socket_state::connected && state_ != fz::socket_state::shutting_down && state_ != fz::socket_state::shut_down) {
 		error = ENOTCONN;
 		return -1;
 	}
-	else if (m_lastReadFailed) {
-		error = EAGAIN;
-		return -1;
-	}
-
-	if (peekBuffer_) {
-		auto min = std::min(static_cast<size_t>(len), peekBuffer_.size());
-		memcpy(buffer, peekBuffer_.get(), min);
-		peekBuffer_.consume(min);
-		error = 0;
-		return static_cast<int>(min);
-	}
 
 	int res = DoCallGnutlsRecordRecv(buffer, len);
 	if (res >= 0) {
@@ -714,7 +658,6 @@ int CTlsSocketImpl::Read(void *buffer, unsigned int len, int& error)
 
 	if (res == GNUTLS_E_INTERRUPTED || res == GNUTLS_E_AGAIN) {
 		error = EAGAIN;
-		m_lastReadFailed = true;
 	}
 	else {
 		Failure(res, false, L"gnutls_record_recv");
@@ -724,20 +667,17 @@ int CTlsSocketImpl::Read(void *buffer, unsigned int len, int& error)
 	return -1;
 }
 
-int CTlsSocketImpl::Write(const void *buffer, unsigned int len, int& error)
+int CTlsSocketImpl::write(void const* buffer, unsigned int len, int& error)
 {
-	if (m_tlsState == CTlsSocket::TlsState::handshake || m_tlsState == CTlsSocket::TlsState::verifycert) {
+	if (state_ == fz::socket_state::connecting) {
 		error = EAGAIN;
 		return -1;
 	}
-	else if (m_tlsState == CTlsSocket::TlsState::closing) {
-		error = EAGAIN;
+	else if (state_ == fz::socket_state::shutting_down || state_ == fz::socket_state::shut_down) {
+		error = ESHUTDOWN;
 		return -1;
 	}
-	else if (m_tlsState == CTlsSocket::TlsState::closed) {
-		return 0;
-	}
-	else if (m_tlsState != CTlsSocket::TlsState::conn) {
+	else if (state_ != fz::socket_state::connected) {
 		error = ENOTCONN;
 		return -1;
 	}
@@ -800,23 +740,23 @@ void CTlsSocketImpl::Failure(int code, bool send_close, std::wstring const& func
 #endif
 				)
 			{
-				if (m_tlsState != CTlsSocket::TlsState::closed || !shutdown_silence_read_errors_) {
+				if (state_ != fz::socket_state::shut_down || !shutdown_silence_read_errors_) {
 					m_pOwner->LogMessage(MessageType::Status, _("Server did not properly shut down TLS connection"));
 				}
 			}
 		}
 	}
 
-	auto const oldState = m_tlsState;
+	auto const oldState = state_;
 
 	Uninit();
 
-	if (send_close) {
+	if (send_close && tlsSocket_.m_pEvtHandler) {
 		int error = m_socket_error;
 		if (!error) {
 			error = ECONNABORTED;
 		}
-		if (oldState == CTlsSocket::TlsState::handshake) {
+		if (oldState == fz::socket_state::connecting) {
 			tlsSocket_.m_pEvtHandler->send_event<fz::socket_event>(&tlsSocket_, fz::socket_event_flag::connection, error);
 		}
 		else {
@@ -825,104 +765,77 @@ void CTlsSocketImpl::Failure(int code, bool send_close, std::wstring const& func
 	}
 }
 
-int CTlsSocketImpl::Peek(void *buffer, unsigned int len, int& error)
-{
-	if (peekBuffer_) {
-		auto min = std::min(static_cast<size_t>(len), peekBuffer_.size());
-		memcpy(buffer, peekBuffer_.get(), min);
-
-		error = 0;
-		return static_cast<int>(min);
-	}
-
-	int read = Read(peekBuffer_.get(len), len, error);
-	if (read <= 0) {
-		return read;
-	}
-	peekBuffer_.add(static_cast<size_t>(read));
-	memcpy(buffer, peekBuffer_.get(), static_cast<size_t>(read));
-
-	return read;
-}
-
-int CTlsSocketImpl::Shutdown(bool silenceReadErrors)
+int CTlsSocketImpl::shutdown()
 {
 	m_pOwner->LogMessage(MessageType::Debug_Verbose, L"CTlsSocketImpl::Shutdown()");
 
-	if (m_tlsState == CTlsSocket::TlsState::closed) {
+	if (state_ == fz::socket_state::shut_down) {
 		return 0;
 	}
-
-	if (m_tlsState == CTlsSocket::TlsState::closing) {
+	else if (state_ == fz::socket_state::shutting_down) {
 		return EAGAIN;
 	}
-
-	if (m_tlsState == CTlsSocket::TlsState::handshake || m_tlsState == CTlsSocket::TlsState::verifycert) {
-		// Shutdown during handshake is not a good idea.
-		m_pOwner->LogMessage(MessageType::Debug_Verbose, L"Shutdown during handshake, postponing");
-		shutdown_requested_ = true;
-		shutdown_silence_read_errors_ = silenceReadErrors;
-		return EAGAIN;
+	else if (state_ != fz::socket_state::connected) {
+		return ENOTCONN;
 	}
 
-	if (m_tlsState != CTlsSocket::TlsState::conn) {
-		return ECONNABORTED;
-	}
+	state_ = fz::socket_state::shutting_down;
 
-	m_tlsState = CTlsSocket::TlsState::closing;
-	shutdown_silence_read_errors_ = silenceReadErrors;
-
-	int res = gnutls_bye(m_session, GNUTLS_SHUT_WR);
-	while ((res == GNUTLS_E_INTERRUPTED || res == GNUTLS_E_AGAIN) && m_canWriteToSocket) {
-		res = gnutls_bye(m_session, GNUTLS_SHUT_WR);
-	}
-	if (!res) {
-		m_tlsState = CTlsSocket::TlsState::closed;
-		return 0;
-	}
-
-	if (res == GNUTLS_E_INTERRUPTED || res == GNUTLS_E_AGAIN) {
-		return EAGAIN;
-	}
-
-	Failure(res, false);
-	return m_socket_error;
+	return ContinueShutdown();
 }
 
-void CTlsSocketImpl::ContinueShutdown()
+int CTlsSocketImpl::ContinueShutdown()
 {
 	m_pOwner->LogMessage(MessageType::Debug_Verbose, L"CTlsSocketImpl::ContinueShutdown()");
 
-	int res = gnutls_bye(m_session, GNUTLS_SHUT_WR);
-	while ((res == GNUTLS_E_INTERRUPTED || res == GNUTLS_E_AGAIN) && m_canWriteToSocket) {
-		res = gnutls_bye(m_session, GNUTLS_SHUT_WR);
+	if (!sent_closure_alert_) {
+		int res = gnutls_bye(m_session, GNUTLS_SHUT_WR);
+		while ((res == GNUTLS_E_INTERRUPTED || res == GNUTLS_E_AGAIN) && m_canWriteToSocket) {
+			res = gnutls_bye(m_session, GNUTLS_SHUT_WR);
+		}
+		if (res == GNUTLS_E_INTERRUPTED || res == GNUTLS_E_AGAIN) {
+			return EAGAIN;
+		}
+		else if (res) {
+			Failure(res, false, L"gnutls_bye");
+			return m_socket_error ? m_socket_error : ECONNABORTED;
+		}
+		sent_closure_alert_ = true;
 	}
+	
+	int res = tlsSocket_.next_layer_.shutdown();
+	if (res == EAGAIN) {
+		return EAGAIN;
+	}
+
 	if (!res) {
-		m_tlsState = CTlsSocket::TlsState::closed;
-
-		tlsSocket_.m_pEvtHandler->send_event<fz::socket_event>(&tlsSocket_, fz::socket_event_flag::write, 0);
-
-		return;
+		state_ = fz::socket_state::shut_down;
 	}
-
-	if (res != GNUTLS_E_INTERRUPTED && res != GNUTLS_E_AGAIN) {
-		Failure(res, true);
+	else {
+		m_socket_error = res;
+		Failure(0, false);
 	}
+	return res;
 }
 
 void CTlsSocketImpl::TrustCurrentCert(bool trusted)
 {
-	if (m_tlsState != CTlsSocket::TlsState::verifycert) {
+	if (state_ != fz::socket_state::connecting && !handshake_successful_) {
 		m_pOwner->LogMessage(MessageType::Debug_Warning, L"TrustCurrentCert called at wrong time.");
 		return;
 	}
 
 	if (trusted) {
-		m_tlsState = CTlsSocket::TlsState::conn;
-		tlsSocket_.m_pEvtHandler->send_event<fz::socket_event>(&tlsSocket_, fz::socket_event_flag::connection, 0);
-		tlsSocket_.m_pEvtHandler->send_event<fz::socket_event>(&tlsSocket_, fz::socket_event_flag::read, 0);
-		if (!shutdown_requested_) {
-			tlsSocket_.m_pEvtHandler->send_event<fz::socket_event>(&tlsSocket_, fz::socket_event_flag::write, 0);
+		state_ = fz::socket_state::connected;
+
+		if (tlsSocket_.m_pEvtHandler) {
+			tlsSocket_.m_pEvtHandler->send_event<fz::socket_event>(&tlsSocket_, fz::socket_event_flag::connection, 0);
+			if (m_canReadFromSocket) {
+				tlsSocket_.m_pEvtHandler->send_event<fz::socket_event>(&tlsSocket_, fz::socket_event_flag::read, 0);
+			}
+			if (m_canWriteToSocket) {
+				tlsSocket_.m_pEvtHandler->send_event<fz::socket_event>(&tlsSocket_, fz::socket_event_flag::write, 0);
+			}
 		}
 
 		return;
@@ -932,7 +845,7 @@ void CTlsSocketImpl::TrustCurrentCert(bool trusted)
 	Failure(0, true);
 }
 
-static std::wstring bin2hex(const unsigned char* in, size_t size)
+static std::wstring bin2hex(unsigned char const* in, size_t size)
 {
 	std::wstring str;
 	str.reserve(size * 3);
@@ -967,7 +880,7 @@ bool CTlsSocketImpl::ExtractCert(gnutls_x509_crt_t const& cert, CCertificate& ou
 	int pkAlgo = gnutls_x509_crt_get_pk_algorithm(cert, &pkBits);
 	std::wstring pkAlgoName;
 	if (pkAlgo >= 0) {
-		const char* pAlgo = gnutls_pk_algorithm_get_name((gnutls_pk_algorithm_t)pkAlgo);
+		char const* pAlgo = gnutls_pk_algorithm_get_name((gnutls_pk_algorithm_t)pkAlgo);
 		if (pAlgo) {
 			pkAlgoName = fz::to_wstring_from_utf8(pAlgo);
 		}
@@ -976,7 +889,7 @@ bool CTlsSocketImpl::ExtractCert(gnutls_x509_crt_t const& cert, CCertificate& ou
 	int signAlgo = gnutls_x509_crt_get_signature_algorithm(cert);
 	std::wstring signAlgoName;
 	if (signAlgo >= 0) {
-		const char* pAlgo = gnutls_sign_algorithm_get_name((gnutls_sign_algorithm_t)signAlgo);
+		char const* pAlgo = gnutls_sign_algorithm_get_name((gnutls_sign_algorithm_t)signAlgo);
 		if (pAlgo) {
 			signAlgoName = fz::to_wstring_from_utf8(pAlgo);
 		}
@@ -984,22 +897,12 @@ bool CTlsSocketImpl::ExtractCert(gnutls_x509_crt_t const& cert, CCertificate& ou
 
 	std::wstring subject, issuer;
 
-	size = 0;
-	res = gnutls_x509_crt_get_dn(cert, nullptr, &size);
-	if (size) {
-		char* dn = new char[size + 1];
-		dn[size] = 0;
-		if (!(res = gnutls_x509_crt_get_dn(cert, dn, &size))) {
-			dn[size] = 0;
-			subject = fz::to_wstring_from_utf8(dn);
-		}
-		else {
-			LogError(res, L"gnutls_x509_crt_get_dn");
-		}
-		delete [] dn;
+	datum_holder raw_subject;
+	if (!gnutls_x509_crt_get_dn3(cert, &raw_subject, 0)) {
+		subject = fz::to_wstring_from_utf8(raw_subject.to_string());
 	}
 	else {
-		LogError(res, L"gnutls_x509_crt_get_dn");
+		LogError(res, L"gnutls_x509_crt_get_dn3");
 	}
 	if (subject.empty()) {
 		m_pOwner->LogMessage(MessageType::Error, _("Could not get distinguished name of certificate subject, gnutls_x509_get_dn failed"));
@@ -1008,22 +911,12 @@ bool CTlsSocketImpl::ExtractCert(gnutls_x509_crt_t const& cert, CCertificate& ou
 
 	std::vector<CCertificate::SubjectName> alt_subject_names = GetCertSubjectAltNames(cert);
 
-	size = 0;
-	res = gnutls_x509_crt_get_issuer_dn(cert, nullptr, &size);
-	if (size) {
-		char* dn = new char[++size + 1];
-		dn[size] = 0;
-		if (!(res = gnutls_x509_crt_get_issuer_dn(cert, dn, &size))) {
-			dn[size] = 0;
-			issuer = fz::to_wstring_from_utf8(dn);
-		}
-		else {
-			LogError(res, L"gnutls_x509_crt_get_issuer_dn");
-		}
-		delete [] dn;
+	datum_holder raw_issuer;
+	if (!gnutls_x509_crt_get_issuer_dn3(cert, &raw_issuer, 0)) {
+		issuer = fz::to_wstring_from_utf8(raw_issuer.to_string());
 	}
 	else {
-		LogError(res, L"gnutls_x509_crt_get_issuer_dn");
+		LogError(res, L"gnutls_x509_crt_get_issuer_dn3");
 	}
 	if (issuer.empty() ) {
 		m_pOwner->LogMessage(MessageType::Error, _("Could not get distinguished name of certificate issuer, gnutls_x509_get_issuer_dn failed"));
@@ -1212,6 +1105,8 @@ bool CTlsSocketImpl::GetSortedPeerCertificates(gnutls_x509_crt_t *& certs, unsig
 	delete[] concated_certs.data;
 
 	if (res != GNUTLS_E_SUCCESS) {
+		certs = nullptr;
+		certs_size = 0;
 		m_pOwner->LogMessage(MessageType::Error, _("Could not sort peer certificates"));
 		return false;
 	}
@@ -1264,23 +1159,21 @@ void CTlsSocketImpl::PrintVerificationError(int status)
 
 int CTlsSocketImpl::VerifyCertificate()
 {
-	if (m_tlsState != CTlsSocket::TlsState::handshake) {
+	if (state_ != fz::socket_state::connecting) {
 		m_pOwner->LogMessage(MessageType::Debug_Warning, L"VerifyCertificate called at wrong time");
-		return FZ_REPLY_ERROR;
+		return ENOTCONN;
 	}
 
-	m_tlsState = CTlsSocket::TlsState::verifycert;
-
 	if (gnutls_certificate_type_get(m_session) != GNUTLS_CRT_X509) {
 		m_pOwner->LogMessage(MessageType::Error, _("Unsupported certificate type"));
 		Failure(0, true);
-		return FZ_REPLY_ERROR;
+		return EOPNOTSUPP;
 	}
 
 	cert_list_holder certs;
 	if (!GetSortedPeerCertificates(certs.certs, certs.certs_size)) {
 		Failure(0, true);
-		return FZ_REPLY_ERROR;
+		return EINVAL;
 	}
 
 	bool const uses_hostname = !hostname_.empty() && fz::get_address_type(hostname_) == fz::address_type::unknown;
@@ -1317,7 +1210,7 @@ int CTlsSocketImpl::VerifyCertificate()
 				m_pOwner->LogMessage(MessageType::Debug_Warning, L"gnutls_certificate_verify_peers2 returned %d with status %u", verifyResult, status);
 				m_pOwner->LogMessage(MessageType::Error, _("Failed to verify peer certificate"));
 				Failure(0, true);
-				return FZ_REPLY_ERROR;
+				return EINVAL;
 			}
 
 			if (!status) {
@@ -1341,7 +1234,7 @@ int CTlsSocketImpl::VerifyCertificate()
 		if (!root) {
 			m_pOwner->LogMessage(MessageType::Error, _("Could not copy certificate"));
 			Failure(0, true);
-			return FZ_REPLY_ERROR;
+			return ECONNABORTED;
 		}
 
 		gnutls_x509_trust_list_t tlist;
@@ -1349,7 +1242,7 @@ int CTlsSocketImpl::VerifyCertificate()
 		if (gnutls_x509_trust_list_add_cas(tlist, &root, 1, 0) != 1) {
 			m_pOwner->LogMessage(MessageType::Error, _("Could not add certificate to temporary trust list"));
 			Failure(0, true);
-			return FZ_REPLY_ERROR;
+			return ECONNABORTED;
 		}
 
 		// 2. Also disable time checks. We allow expired/not yet valid certificates, though only
@@ -1363,14 +1256,14 @@ int CTlsSocketImpl::VerifyCertificate()
 			m_pOwner->LogMessage(MessageType::Debug_Warning, L"gnutls_certificate_verify_peers2 returned %d with status %u", verifyResult, status);
 			m_pOwner->LogMessage(MessageType::Error, _("Failed to verify peer certificate"));
 			Failure(0, true);
-			return FZ_REPLY_ERROR;
+			return EINVAL;
 		}
 
 		if (status != 0) {
 			PrintVerificationError(status);
 
 			Failure(0, true);
-			return FZ_REPLY_ERROR;
+			return EINVAL;
 		}
 
 		// 3. Hostname mismatch
@@ -1385,24 +1278,24 @@ int CTlsSocketImpl::VerifyCertificate()
 	datum_holder cert_der{};
 	if (gnutls_x509_crt_export2(certs.certs[0], GNUTLS_X509_FMT_DER, &cert_der) != GNUTLS_E_SUCCESS) {
 		Failure(0, true);
-		return FZ_REPLY_ERROR;
+		return ECONNABORTED;
 	}
 
-	if (m_implicitTrustedCert.data) {
-		if (m_implicitTrustedCert.size != cert_der.size ||
-			memcmp(m_implicitTrustedCert.data, cert_der.data, cert_der.size))
+	if (!required_certificate_.empty()) {
+		if (required_certificate_.size() != cert_der.size ||
+			memcmp(required_certificate_.data(), cert_der.data, cert_der.size))
 		{
-			m_pOwner->LogMessage(MessageType::Error, _("Primary connection and data connection certificates don't match."));
+			m_pOwner->LogMessage(MessageType::Error, _("Certificate of connection does not match expected certificate."));
 			Failure(0, true);
-			return FZ_REPLY_ERROR;
+			return EINVAL;
 		}
 
 		TrustCurrentCert(true);
 
-		if (m_tlsState != CTlsSocket::TlsState::conn) {
-			return FZ_REPLY_ERROR;
+		if (state_ != fz::socket_state::connected && state_ != fz::socket_state::shutting_down && state_ != fz::socket_state::shut_down) {
+			return ECONNABORTED;
 		}
-		return FZ_REPLY_OK;
+		return 0;
 	}
 
 	m_pOwner->LogMessage(MessageType::Status, _("Verifying certificate..."));
@@ -1416,20 +1309,28 @@ int CTlsSocketImpl::VerifyCertificate()
 		}
 		else {
 			Failure(0, true);
-			return FZ_REPLY_ERROR;
+			return ECONNABORTED;
 		}
 	}
 
 	if (CertificateIsBlacklisted(certificates)) {
 		Failure(0, true);
-		return FZ_REPLY_ERROR;
+		return EINVAL;
 	}
 
 	int const algorithmWarnings = GetAlgorithmWarnings();
 
+	int error;
+	auto port = tlsSocket_.peer_port(error);
+	if (port == -1) {
+		m_socket_error = error;
+		Failure(0, true);
+		return ECONNABORTED;
+	}
+
 	CCertificateNotification *pNotification = new CCertificateNotification(
 		fz::to_wstring(hostname_),
-		port_,
+		port,
 		GetProtocolName(),
 		GetKeyExchange(),
 		GetCipherName(),
@@ -1442,18 +1343,14 @@ int CTlsSocketImpl::VerifyCertificate()
 	// Finally, ask user to verify the certificate chain
 	m_pOwner->SendAsyncRequest(pNotification);
 
-	return FZ_REPLY_WOULDBLOCK;
-}
-
-void CTlsSocketImpl::OnRateAvailable(CRateLimiter::rate_direction)
-{
+	return EAGAIN;
 }
 
 std::wstring CTlsSocketImpl::GetProtocolName()
 {
 	std::wstring ret;
 
-	const char* s = gnutls_protocol_get_name( gnutls_protocol_get_version( m_session ) );
+	char const* s = gnutls_protocol_get_name( gnutls_protocol_get_version( m_session ) );
 	if (s && *s) {
 		ret = fz::to_wstring_from_utf8(s);
 	}
@@ -1469,7 +1366,7 @@ std::wstring CTlsSocketImpl::GetKeyExchange()
 {
 	std::wstring ret;
 
-	const char* s = gnutls_kx_get_name( gnutls_kx_get( m_session ) );
+	char const* s = gnutls_kx_get_name( gnutls_kx_get( m_session ) );
 	if (s && *s) {
 		ret = fz::to_wstring_from_utf8(s);
 	}
@@ -1485,7 +1382,7 @@ std::wstring CTlsSocketImpl::GetCipherName()
 {
 	std::wstring ret;
 
-	const char* cipher = gnutls_cipher_get_name(gnutls_cipher_get(m_session));
+	char const* cipher = gnutls_cipher_get_name(gnutls_cipher_get(m_session));
 	if (cipher && *cipher) {
 		ret = fz::to_wstring_from_utf8(cipher);
 	}
@@ -1501,7 +1398,7 @@ std::wstring CTlsSocketImpl::GetMacName()
 {
 	std::wstring ret;
 
-	const char* mac = gnutls_mac_get_name(gnutls_mac_get(m_session));
+	char const* mac = gnutls_mac_get_name(gnutls_mac_get(m_session));
 	if (mac && *mac) {
 		ret = fz::to_wstring_from_utf8(mac);
 	}
@@ -1522,7 +1419,7 @@ std::string CTlsSocketImpl::ListTlsCiphers(std::string priority)
 	auto list = fz::sprintf("Ciphers for %s:\n", priority);
 
 	gnutls_priority_t pcache;
-	const char *err = nullptr;
+	char const* err = nullptr;
 	int ret = gnutls_priority_init(&pcache, priority.c_str(), &err);
 	if (ret < 0) {
 		list += fz::sprintf("gnutls_priority_init failed with code %d: %s", ret, err ? err : "Unknown error");
@@ -1541,7 +1438,7 @@ std::string CTlsSocketImpl::ListTlsCiphers(std::string priority)
 
 			gnutls_protocol_t version;
 			unsigned char id[2];
-			const char* name = gnutls_cipher_suite_info(idx, id, nullptr, nullptr, nullptr, &version);
+			char const* name = gnutls_cipher_suite_info(idx, id, nullptr, nullptr, nullptr, &version);
 
 			if (name != nullptr) {
 				list += fz::sprintf(
@@ -1560,7 +1457,7 @@ std::string CTlsSocketImpl::ListTlsCiphers(std::string priority)
 int CTlsSocketImpl::DoCallGnutlsRecordRecv(void* data, size_t len)
 {
 	ssize_t res = gnutls_record_recv(m_session, data, len);
-	while( (res == GNUTLS_E_AGAIN || res == GNUTLS_E_INTERRUPTED) && m_canReadFromSocket && !gnutls_record_get_direction(m_session)) {
+	while ((res == GNUTLS_E_AGAIN || res == GNUTLS_E_INTERRUPTED) && m_canReadFromSocket && !gnutls_record_get_direction(m_session)) {
 		// Spurious EAGAIN. Can happen if GnuTLS gets a partial
 		// record and the socket got closed.
 		// The unexpected close is being ignored in this case, unless
@@ -1576,10 +1473,61 @@ int CTlsSocketImpl::DoCallGnutlsRecordRecv(void* data, size_t len)
 
 std::wstring CTlsSocketImpl::GetGnutlsVersion()
 {
-	const char* v = gnutls_check_version(nullptr);
+	char const* v = gnutls_check_version(nullptr);
 	if (!v || !*v) {
 		return L"unknown";
 	}
 
 	return fz::to_wstring(v);
 }
+
+void CTlsSocketImpl::set_hostname(fz::native_string const& host)
+{
+	hostname_ = host;
+	if (!hostname_.empty() && fz::get_address_type(hostname_) == fz::address_type::unknown) {
+		auto const utf8 = fz::to_utf8(hostname_);
+		if (!utf8.empty()) {
+			int res = gnutls_server_name_set(m_session, GNUTLS_NAME_DNS, utf8.c_str(), utf8.size());
+			if (res) {
+				LogError(res, L"gnutls_server_name_set", MessageType::Debug_Warning);
+			}
+		}
+	}
+}
+
+int CTlsSocketImpl::connect(fz::native_string const& host, unsigned int port, fz::address_type family)
+{
+	set_hostname(host);
+
+	return tlsSocket_.next_layer_.connect(host, port, family);
+}
+
+std::vector<uint8_t> CTlsSocketImpl::get_session_parameters() const
+{
+	std::vector<uint8_t> ret;
+
+	datum_holder d;
+	int res = gnutls_session_get_data2(m_session, &d);
+	if (res) {
+		m_pOwner->LogMessage(MessageType::Debug_Warning, L"gnutls_session_get_data2 failed: %d", res);
+	}
+	else {
+		ret.assign(d.data, d.data + d.size);
+	}
+	
+	return ret;
+}
+
+std::vector<uint8_t> CTlsSocketImpl::get_raw_certificate() const
+{
+	std::vector<uint8_t> ret;
+
+	// Implicitly trust certificate of primary socket
+	unsigned int cert_list_size;
+	gnutls_datum_t const* const cert_list = gnutls_certificate_get_peers(m_session, &cert_list_size);
+	if (cert_list && cert_list_size) {
+		ret.assign(cert_list[0].data, cert_list[0].data + cert_list[0].size);
+	}
+
+	return ret;
+}
