@@ -1149,7 +1149,6 @@ bool CQueueView::RemoveItem(CQueueItem* item, bool destroy, bool updateItemCount
 			if (updateItemCount) {
 				DisplayQueueSize();
 			}
-			wxASSERT(m_totalQueueSize >= 0);
 		}
 	}
 
@@ -1720,7 +1719,7 @@ void CQueueView::ImportQueue(pugi::xml_node element, bool updateSelections)
 						continue;
 					}
 
-					// CServerPath and wxString are reference counted.
+					// CServerPath and CLocalPath are reference counted.
 					// Save some memory here by re-using the old copy
 					if (localPath != previousLocalPath) {
 						previousLocalPath = localPath;
@@ -1749,10 +1748,11 @@ void CQueueView::ImportQueue(pugi::xml_node element, bool updateSelections)
 				bool download = GetTextElementInt(folder, "Download") != 0;
 				if (download) {
 					std::wstring localFile = GetTextElement(folder, "LocalFile");
-					if (localFile.empty()) {
+					CLocalPath localPath(localFile);
+					if (localPath.empty()) {
 						continue;
 					}
-					folderItem = new CFolderItem(pServerItem, true, CLocalPath(localFile));
+					folderItem = new CFolderItem(pServerItem, true, localPath);
 				}
 				else {
 					std::wstring remoteFile = GetTextElement(folder, "RemoteFile");
@@ -2438,13 +2438,7 @@ void CQueueView::UpdateItemSize(CFileItem* pItem, int64_t size)
 		}
 	}
 	else {
-		wxASSERT(m_totalQueueSize >= oldSize);
-		if (m_totalQueueSize > oldSize) {
-			m_totalQueueSize -= oldSize;
-		}
-		else {
-			m_totalQueueSize = 0;
-		}
+		m_totalQueueSize -= oldSize;
 	}
 
 	if (size < 0) {
@@ -3196,27 +3190,30 @@ void CQueueView::OnStateChange(CState*, t_statechange_notifications notification
 			return;
 		}
 
-
+		bool const forget = COptions::Get()->GetOptionVal(OPTION_DEFAULT_KIOSKMODE) != 0;
 		for (auto it = m_serverList.begin(); it != m_serverList.end(); ) {
 			Site site = (*it)->GetSite();
-			loginManager->AskDecryptor(site.credentials.encrypted_, true, false);
+			if (!forget) {
+				loginManager->AskDecryptor(site.credentials.encrypted_, true, false);
 
-			// Since the queue may be running and AskDecryptor uses the GUI, re-find matching server item
-			for (it = m_serverList.begin(); it != m_serverList.end(); ++it) {
-				if ((*it)->GetSite() == site) {
-					site = (*it)->GetSite(); // Credentials aren't in ==
-					site.credentials.Unprotect(loginManager->GetDecryptor(site.credentials.encrypted_), true);
-					(*it)->GetCredentials() = site.credentials;
-					break;
+				// Since the queue may be running and AskDecryptor uses the GUI, re-find matching server item
+				for (it = m_serverList.begin(); it != m_serverList.end(); ++it) {
+					if ((*it)->GetSite() == site) {
+						site = (*it)->GetSite(); // Credentials aren't in ==
+						site.credentials.Unprotect(loginManager->GetDecryptor(site.credentials.encrypted_), true);
+						(*it)->GetCredentials() = site.credentials;
+						break;
+					}
+				}
+				if (it == m_serverList.end()) {
+					// Server has vanished, start over.
+					it = m_serverList.begin();
+					continue;
 				}
 			}
-			if (it == m_serverList.end()) {
-				// Server has vanished, start over.
-				it = m_serverList.begin();
-			}
-			else {
-				++it;
-			}
+
+			(*it)->GetCredentials().Protect();
+			++it;
 		}
 	}
 	else if (notification == STATECHANGE_QUITNOW) {
