@@ -1,11 +1,12 @@
-#ifndef __NETCONFWIZARD_H__
-#define __NETCONFWIZARD_H__
+#ifndef FILEZILLA_INTERFACES_NETCONFWIZARD_HEADER
+#define FILEZILLA_INTERFACES_NETCONFWIZARD_HEADER
 
 #include <wx/wizard.h>
 #include "wrapengine.h"
 #include "externalipresolver.h"
 #include <wx/timer.h>
-#include "socket.h"
+
+#include <libfilezilla/socket.hpp>
 
 #define NETCONFBUFFERSIZE 200
 
@@ -69,7 +70,7 @@ protected:
 	wxString m_nextLabelText;
 
 	// Test data
-	fz::socket* m_socket{};
+	std::unique_ptr<fz::socket> socket_;
 	int m_state;
 
 	char m_recvBuffer[NETCONFBUFFERSIZE];
@@ -92,8 +93,8 @@ protected:
 	CExternalIPResolver* m_pIPResolver{};
 	wxString m_externalIP;
 
-	fz::socket* m_pSocketServer{};
-	fz::socket* m_pDataSocket{};
+	std::unique_ptr<fz::listen_socket> listen_socket_;
+	std::unique_ptr<fz::socket> data_socket_;
 	int m_listenPort{};
 	bool gotListReply{};
 	int m_data{};
@@ -103,4 +104,4 @@ protected:
 	wxTimer m_timer;
 };
 
-#endif //__NETCONFWIZARD_H__
+#endif
