@@ -1,12 +1,12 @@
-#ifndef __QUEUE_H__
-#define __QUEUE_H__
+#ifndef FILEZILLA_INTERFACE_QUEUE_HEADER
+#define FILEZILLA_INTERFACE_QUEUE_HEADER
 
 #include "aui_notebook_ex.h"
 #include "listctrlex.h"
 #include "edithandler.h"
 #include <libfilezilla/optional.hpp>
 
-enum class QueuePriority : char {
+enum class QueuePriority : unsigned char {
 	lowest,
 	low,
 	normal,
@@ -78,14 +78,15 @@ private:
 };
 
 class CFileItem;
-class CServerItem : public CQueueItem
+class CServerItem final : public CQueueItem
 {
 public:
-	CServerItem(const CServer& server);
+	CServerItem(Site const& site);
 	virtual ~CServerItem();
 	virtual QueueItemType GetType() const { return QueueItemType::Server; }
 
-	const CServer& GetServer() const;
+	Site const& GetSite() const { return site_; }
+	ProtectedCredentials& GetCredentials() {return site_.credentials; }
 	wxString GetName() const;
 
 	virtual void AddChild(CQueueItem* pItem);
@@ -95,7 +96,7 @@ public:
 	CFileItem* GetIdleChild(bool immadiateOnly, TransferDirection direction);
 
 	virtual bool RemoveChild(CQueueItem* pItem, bool destroy = true, bool forward = true); // Removes a child item with is somewhere in the tree of children
-	virtual bool TryRemoveAll();
+	virtual bool TryRemoveAll() override;
 
 	int64_t GetTotalSize(int& filesWithUnknownSize, int& queuedFiles) const;
 
@@ -116,11 +117,13 @@ public:
 
 	const std::vector<CQueueItem*>& GetChildren() const { return m_children; }
 
+	void Sort(int col, bool reverse);
+
 protected:
 	void AddFileItemToList(CFileItem* pItem);
 	void RemoveFileItemFromList(CFileItem* pItem, bool forward);
 
-	CServer m_server;
+	Site site_;
 
 	// array of item lists, sorted by priority. Used by scheduler to find
 	// next file to transfer
@@ -168,19 +171,23 @@ public:
 	inline bool queued() const { return (flags & flag_queued) != 0; }
 	inline void set_queued(bool q)
 	{
-		if (q)
+		if (q) {
 			flags |= flag_queued;
-		else
+		}
+		else {
 			flags &= ~flag_queued;
+		}
 	}
 
 	inline bool pending_remove() const { return (flags & flag_remove) != 0; }
 	inline void set_pending_remove(bool remove)
 	{
-		if (remove)
+		if (remove) {
 			flags |= flag_remove;
-		else
+		}
+		else {
 			flags &= ~flag_remove;
+		}
 	}
 
 	virtual QueueItemType GetType() const { return QueueItemType::File; }
@@ -190,12 +197,13 @@ public:
 
 	virtual void SaveItem(pugi::xml_node& element) const;
 
-	virtual bool TryRemoveAll(); // Removes inactive children, queues active children for removal.
-								 // Returns true if item can be removed itself
+	// Removes inactive children, queues active children for removal.
+	// Returns true if item can be removed itself
+	virtual bool TryRemoveAll() override final;
 
 	void SetTargetFile(wxString const& file);
 
-	enum Status : char {
+	enum class Status : unsigned char {
 		none,
 		incorrect_password,
 		timeout,
@@ -241,10 +249,12 @@ public:
 	inline bool made_progress() const { return (flags & flag_made_progress) != 0; }
 	inline void set_made_progress(bool made_progress)
 	{
-		if (made_progress)
+		if (made_progress) {
 			flags |= flag_made_progress;
-		else
+		}
+		else {
 			flags &= ~flag_made_progress;
+		}
 	}
 
 	bool Ascii() const { return (flags & flag_ascii) != 0; }
@@ -267,7 +277,7 @@ protected:
 	int64_t m_size{};
 };
 
-class CFolderItem : public CFileItem
+class CFolderItem final : public CFileItem
 {
 public:
 	CFolderItem(CServerItem* parent, bool queued, CLocalPath const& localPath);
@@ -278,19 +288,17 @@ public:
 	virtual void SaveItem(pugi::xml_node& element) const;
 
 	virtual void SetActive(bool active);
-
-	virtual bool TryRemoveAll() { return true; }
 };
 
-class CStatusItem : public CQueueItem
+class CStatusItem final : public CQueueItem
 {
 public:
-	CStatusItem() {}
-	virtual ~CStatusItem() {}
+	CStatusItem() = default;
+	virtual ~CStatusItem() = default;
 
 	virtual QueueItemType GetType() const { return QueueItemType::Status; }
 
-	virtual bool TryRemoveAll() { return true; }
+	virtual bool TryRemoveAll() override { return true; }
 };
 
 class CQueue;
@@ -314,7 +322,7 @@ public:
 	virtual ~CQueueViewBase();
 
 	// Gets item for given server or creates new if it doesn't exist
-	CServerItem* CreateServerItem(const CServer& server);
+	CServerItem* CreateServerItem(Site const& site);
 
 	virtual void InsertItem(CServerItem* pServerItem, CQueueItem* pItem);
 	virtual bool RemoveItem(CQueueItem* pItem, bool destroy, bool updateItemCount = true, bool updateSelections = true, bool forward = true);
@@ -331,11 +339,11 @@ public:
 
 protected:
 
-	void CreateColumns(std::list<ColumnId> const& extraColumns = std::list<ColumnId>());
+	void CreateColumns(std::vector<ColumnId> const& extraColumns = std::vector<ColumnId>());
 	void AddQueueColumn(ColumnId id);
 
 	// Gets item for given server
-	CServerItem* GetServerItem(const CServer& server);
+	CServerItem* GetServerItem(Site const& site);
 
 	// Gets item with given index
 	CQueueItem* GetQueueItem(unsigned int item) const;
@@ -394,7 +402,7 @@ class CQueueViewSuccessful;
 
 class CMainFrame;
 class CAsyncRequestQueue;
-class CQueue : public wxAuiNotebookEx
+class CQueue final : public wxAuiNotebookEx
 {
 public:
 	CQueue(wxWindow* parent, CMainFrame* pMainFrame, CAsyncRequestQueue* pAsyncRequestQueue);
@@ -407,11 +415,11 @@ public:
 	virtual void SetFocus();
 protected:
 
-	CQueueView* m_pQueueView;
-	CQueueViewFailed* m_pQueueView_Failed;
-	CQueueViewSuccessful* m_pQueueView_Successful;
+	CQueueView* m_pQueueView{};
+	CQueueViewFailed* m_pQueueView_Failed{};
+	CQueueViewSuccessful* m_pQueueView_Successful{};
 };
 
 #include "QueueView.h"
 
-#endif //__QUEUE_H__
+#endif
