@@ -58,7 +58,7 @@ bool CSiteManager::Load(pugi::xml_node element, CSiteManagerXmlHandler& handler)
 			}
 
 			const bool expand = GetTextAttribute(child, "expanded") != _T("0");
-			if (!handler.AddFolder(name, expand)) {
+			if (!handler.AddFolder(name.substr(0, 255), expand)) {
 				return false;
 			}
 			Load(child, handler);
@@ -100,10 +100,10 @@ std::unique_ptr<Site> CSiteManager::ReadServerElement(pugi::xml_node element)
 {
 	auto data = std::make_unique<Site>();
 	if (!::GetServer(element, *data)) {
-		return 0;
+		return nullptr;
 	}
 	if (data->server.GetName().empty()) {
-		return 0;
+		return nullptr;
 	}
 
 	data->comments_ = GetTextElement(element, "Comments");
@@ -120,7 +120,7 @@ std::unique_ptr<Site> CSiteManager::ReadServerElement(pugi::xml_node element)
 
 		Bookmark bookmarkData;
 		if (ReadBookmarkElement(bookmarkData, bookmark)) {
-			bookmarkData.m_name = name;
+			bookmarkData.m_name = name.substr(0, 255);
 			data->m_bookmarks.push_back(bookmarkData);
 		}
 	}
@@ -590,7 +590,7 @@ pugi::xml_node CSiteManager::GetElementByPath(pugi::xml_node node, std::vector<s
 				continue;
 			}
 
-			wxString name = GetTextElement_Trimmed(child, "Name");
+			std::wstring name = GetTextElement_Trimmed(child, "Name");
 			if (name.empty()) {
 				name = GetTextElement_Trimmed(child);
 			}
@@ -799,6 +799,7 @@ wxString CSiteManager::GetColourName(int i)
 
 void CSiteManager::Rewrite(CLoginManager & loginManager, pugi::xml_node element, bool on_failure_set_to_ask)
 {
+	bool const forget = COptions::Get()->GetOptionVal(OPTION_DEFAULT_KIOSKMODE) != 0;
 	for (auto child = element.first_child(); child; child = child.next_sibling()) {
 		if (!strcmp(child.name(), "Folder")) {
 			Rewrite(loginManager, child, on_failure_set_to_ask);
@@ -806,8 +807,11 @@ void CSiteManager::Rewrite(CLoginManager & loginManager, pugi::xml_node element,
 		else if (!strcmp(child.name(), "Server")) {
 			auto site = ReadServerElement(child);
 			if (site) {
-				loginManager.AskDecryptor(site->credentials.encrypted_, true, false);
-				site->credentials.Unprotect(loginManager.GetDecryptor(site->credentials.encrypted_), on_failure_set_to_ask);
+				if (!forget) {
+					loginManager.AskDecryptor(site->credentials.encrypted_, true, false);
+					site->credentials.Unprotect(loginManager.GetDecryptor(site->credentials.encrypted_), on_failure_set_to_ask);
+				}
+				site->credentials.Protect();
 				Save(child, *site);
 			}
 		}
@@ -872,3 +876,110 @@ void CSiteManager::Save(pugi::xml_node element, Site const& site)
 		AddTextElementUtf8(node, "DirectoryComparison", bookmark.m_comparison ? "1" : "0");
 	}
 }
+
+namespace {
+pugi::xml_node GetChildWithName(pugi::xml_node element, std::wstring const& name)
+{
+	pugi::xml_node child;
+	for (child = element.first_child(); child; child = child.next_sibling()) {
+		std::wstring childName = GetTextElement_Trimmed(child, "Name");
+		if (childName.empty()) {
+			childName = GetTextElement_Trimmed(child);
+		}
+		if (!fz::stricmp(name, childName)) {
+			return child;
+		}
+	}
+	
+	return pugi::xml_node();
+}
+
+pugi::xml_node GetOrCreateFolderWithName(pugi::xml_node element, std::wstring const& name)
+{
+	pugi::xml_node child = GetChildWithName(element, name);
+	if (child) {
+		if (strcmp(child.name(), "Folder")) {
+			// We do not allow servers and directories to share the same name
+			child = pugi::xml_node();
+		}
+	}
+	else {
+		child = element.append_child("Folder");
+		AddTextElement(child, name);
+	}
+	return child;
+}
+}
+
+bool CSiteManager::ImportSites(pugi::xml_node sites)
+{
+	CInterProcessMutex mutex(MUTEX_SITEMANAGER);
+
+	CXmlFile file(wxGetApp().GetSettingsFile(_T("sitemanager")));
+	auto element = file.Load();
+	if (!element) {
+		wxString msg = wxString::Format(_("Could not load \"%s\", please make sure the file is valid and can be accessed.\nAny changes made in the Site Manager will not be saved."), file.GetFileName());
+		wxMessageBoxEx(msg, _("Error loading xml file"), wxICON_ERROR);
+
+		return false;
+	}
+
+	auto currentSites = element.child("Servers");
+	if (!currentSites) {
+		currentSites = element.append_child("Servers");
+	}
+
+	if (!ImportSites(sites, currentSites)) {
+		return false;
+	}
+
+	return file.Save(true);
+}
+
+bool CSiteManager::ImportSites(pugi::xml_node sitesToImport, pugi::xml_node existingSites)
+{
+	for (auto importFolder = sitesToImport.child("Folder"); importFolder; importFolder = importFolder.next_sibling("Folder")) {
+		std::wstring name = GetTextElement_Trimmed(importFolder, "Name").substr(0, 255);
+		if (name.empty()) {
+			name = GetTextElement_Trimmed(importFolder);
+		}
+		if (name.empty()) {
+			continue;
+		}
+
+		std::wstring newName = name.substr(0, 240);
+		int i = 2;
+		pugi::xml_node folder;
+		while (!(folder = GetOrCreateFolderWithName(existingSites, newName))) {
+			newName = fz::sprintf(L"%s %d", name.substr(0, 240), i++);
+		}
+
+		ImportSites(importFolder, folder);
+	}
+
+	auto & loginManager = CLoginManager::Get();
+
+	for (auto importSite = sitesToImport.child("Server"); importSite; importSite = importSite.next_sibling("Server")) {
+		auto site = ReadServerElement(importSite);
+		if (!site) {
+			continue;
+		}
+			
+		// Find free name
+		auto const name = site->server.GetName();
+		std::wstring newName = name;
+		int i = 2;
+		while (GetChildWithName(existingSites, newName)) {
+			newName = fz::sprintf(L"%s %d", name.substr(0, 240), i++);
+		}
+		site->server.SetName(newName);
+
+		site->credentials.Unprotect(loginManager.GetDecryptor(site->credentials.encrypted_), false);
+		site->credentials.Protect();
+
+		auto xsite = existingSites.append_child("Server");
+		Save(xsite, *site);
+	}
+
+	return true;
+}
