@@ -11,10 +11,9 @@
 
 #include <wx/scrolwin.h>
 #include <wx/statbox.h>
-#include <wx/tokenzr.h>
 
 CertStore::CertStore()
-	: m_xmlFile(wxGetApp().GetSettingsFile(_T("trustedcerts")))
+	: m_xmlFile(wxGetApp().GetSettingsFile(L"trustedcerts"))
 {
 }
 
@@ -407,7 +406,7 @@ bool CVerifyCertDialog::DisplayCert(wxDialogEx* pDlg, const CCertificate& cert)
 	pDlg->SetChildLabel(XRCID("ID_SIGNALGO"), cert.GetSignatureAlgorithm());
 
 	wxString const& sha256 = cert.GetFingerPrintSHA256();
-	pDlg->SetChildLabel(XRCID("ID_FINGERPRINT_SHA256"), sha256.Left(sha256.size() / 2 + 1) + _T("\n") + sha256.Mid(sha256.size() / 2 + 1));
+	pDlg->SetChildLabel(XRCID("ID_FINGERPRINT_SHA256"), sha256.Left(sha256.size() / 2 + 1) + L"\n" + sha256.Mid(sha256.size() / 2 + 1));
 	pDlg->SetChildLabel(XRCID("ID_FINGERPRINT_SHA1"), cert.GetFingerPrintSHA1());
 
 	ParseDN(XRCCTRL(*pDlg, "ID_ISSUER_BOX", wxStaticBox), cert.GetIssuer(), m_pIssuerSizer);
@@ -421,7 +420,7 @@ bool CVerifyCertDialog::DisplayCert(wxDialogEx* pDlg, const CCertificate& cert)
 	if (!altNames.empty()) {
 		wxString str;
 		for (auto const& altName : altNames) {
-			str += altName.name + _T("\n");
+			str += LabelEscape(altName.name) + L"\n";
 		}
 		str.RemoveLast();
 		m_pSubjectSizer->Add(new wxStaticText(subjectPanel, wxID_ANY, wxPLURAL("Alternative name:", "Alternative names:", altNames.size())));
@@ -450,7 +449,7 @@ bool CVerifyCertDialog::DisplayCert(wxDialogEx* pDlg, const CCertificate& cert)
 bool CVerifyCertDialog::DisplayAlgorithm(int controlId, wxString name, bool insecure)
 {
 	if (insecure) {
-		name += _T(" - ");
+		name += L" - ";
 		name += _("Insecure algorithm!");
 
 		auto wnd = m_pDlg->FindWindow(controlId);
@@ -467,7 +466,7 @@ bool CVerifyCertDialog::DisplayAlgorithm(int controlId, wxString name, bool inse
 void CVerifyCertDialog::ShowVerificationDialog(CCertificateNotification& notification, bool displayOnly)
 {
 	m_pDlg = new wxDialogEx;
-	if (!m_pDlg->Load(0, _T("ID_VERIFYCERT"))) {
+	if (!m_pDlg->Load(0, L"ID_VERIFYCERT")) {
 		wxBell();
 		delete m_pDlg;
 		m_pDlg = 0;
@@ -480,7 +479,7 @@ void CVerifyCertDialog::ShowVerificationDialog(CCertificateNotification& notific
 		xrc_call(*m_pDlg, "ID_ALWAYS", &wxWindow::Hide);
 		xrc_call(*m_pDlg, "ID_TRUST_SANS", &wxWindow::Hide);
 		xrc_call(*m_pDlg, "wxID_CANCEL", &wxWindow::Hide);
-		m_pDlg->SetTitle(_T("Certificate details"));
+		m_pDlg->SetTitle(L"Certificate details");
 	}
 	else {
 		m_pDlg->WrapText(m_pDlg, XRCID("ID_DESC"), 420);
@@ -498,7 +497,7 @@ void CVerifyCertDialog::ShowVerificationDialog(CCertificateNotification& notific
 	else {
 		wxChoice* pChoice = XRCCTRL(*m_pDlg, "ID_CHAIN", wxChoice);
 		for (unsigned int i = 0; i < m_certificates.size(); ++i) {
-			pChoice->Append(wxString::Format(_T("%d"), i));
+			pChoice->Append(wxString::Format(L"%d", i));
 		}
 		pChoice->SetSelection(0);
 
@@ -507,10 +506,10 @@ void CVerifyCertDialog::ShowVerificationDialog(CCertificateNotification& notific
 
 	if (notification.MismatchedHostname()) {
 		xrc_call(*m_pDlg, "ID_HOST", &wxWindow::SetForegroundColour, wxColour(255, 0, 0));
-		m_pDlg->SetChildLabel(XRCID("ID_HOST"), wxString::Format(_("%s:%d - Hostname does not match certificate"), notification.GetHost(), notification.GetPort()));
+		m_pDlg->SetChildLabel(XRCID("ID_HOST"), wxString::Format(_("%s:%d - Hostname does not match certificate"), LabelEscape(notification.GetHost()), notification.GetPort()));
 	}
 	else {
-		m_pDlg->SetChildLabel(XRCID("ID_HOST"), wxString::Format(_T("%s:%d"), notification.GetHost(), notification.GetPort()));
+		m_pDlg->SetChildLabel(XRCID("ID_HOST"), wxString::Format(L"%s:%d", LabelEscape(notification.GetHost()), notification.GetPort()));
 	}
 
 	line_height_ = XRCCTRL(*m_pDlg, "ID_SUBJECT_DUMMY", wxStaticText)->GetSize().y;
@@ -579,145 +578,122 @@ void CVerifyCertDialog::ShowVerificationDialog(CCertificateNotification& notific
 	m_pDlg = 0;
 }
 
-void CVerifyCertDialog::ParseDN(wxWindow* parent, const wxString& dn, wxSizer* pSizer)
+namespace {
+std::vector<std::pair<std::wstring, std::wstring>> dn_split(std::wstring const& dn)
+{
+	std::vector<std::pair<std::wstring, std::wstring>> ret;
+
+	std::wstring type;
+	std::wstring value;
+
+	int escaping{};
+	bool phase{};
+
+	for (auto const& c : dn) {
+		auto& out = phase ? value : type;
+		if (escaping) {
+			if ((c >= '0' && c <= '9') || (c >= 'a' && c <= 'z') || (c >= 'A' && c <= 'Z')) {
+				--escaping;
+			}
+			else {
+				escaping = 0;
+			}
+			out += c;
+		}
+		else if (!phase && c == '=') {
+			phase = true;
+		}
+		else if (c == '+' || c == ',') {
+			if (!type.empty() && !value.empty()) {
+				ret.emplace_back(type, value);
+			}
+			type.clear();
+			value.clear();
+			phase = false;
+		}
+		else if (c == '\\') {
+			out += c;
+			escaping = 2;
+		}
+		else {
+			out += c;
+		}
+	}
+
+	if (!type.empty() && !value.empty()) {
+		ret.emplace_back(type, value);
+	}
+
+	return ret;
+}
+}
+
+
+void CVerifyCertDialog::ParseDN(wxWindow* parent, std::wstring const& dn, wxSizer* pSizer)
 {
 	pSizer->Clear(true);
 
-	wxStringTokenizer tokens(dn, _T(","));
+	auto tokens = dn_split(dn);
 
-	std::list<wxString> tokenlist;
-	while (tokens.HasMoreTokens()) {
-		tokenlist.push_back(tokens.GetNextToken());
-	}
+	ParseDN_by_prefix(parent, tokens, L"CN", _("Common name:"), pSizer);
+	ParseDN_by_prefix(parent, tokens, L"O", _("Organization:"), pSizer);
+	ParseDN_by_prefix(parent, tokens, L"businessCategory", _("Business category:"), pSizer);
+	ParseDN_by_prefix(parent, tokens, L"OU", _("Unit:"), pSizer);
+	ParseDN_by_prefix(parent, tokens, L"title", _("Title:"), pSizer);
+	ParseDN_by_prefix(parent, tokens, L"C", _("Country:"), pSizer);
+	ParseDN_by_prefix(parent, tokens, L"ST", _("State or province:"), pSizer);
+	ParseDN_by_prefix(parent, tokens, L"L", _("Locality:"), pSizer);
+	ParseDN_by_prefix(parent, tokens, L"postalCode", _("Postal code:"), pSizer);
+	ParseDN_by_prefix(parent, tokens, L"street", _("Street:"), pSizer);
+	ParseDN_by_prefix(parent, tokens, L"EMAIL", _("E-Mail:"), pSizer);
+	ParseDN_by_prefix(parent, tokens, L"serialNumber", _("Serial number:"), pSizer);
+	ParseDN_by_prefix(parent, tokens, L"telephoneNumber", _("Telephone number:"), pSizer);
+	ParseDN_by_prefix(parent, tokens, L"name", _("Name:"), pSizer);
+	ParseDN_by_prefix(parent, tokens, L"jurisdictionOfIncorporationCountryName", _("Jurisdiction country:"), pSizer);
+	ParseDN_by_prefix(parent, tokens, L"jurisdictionOfIncorporationStateOrProvinceName", _("Jurisdiction state or province:"), pSizer);
+	ParseDN_by_prefix(parent, tokens, L"jurisdictionOfIncorporationLocalityName", _("Jurisdiction locality:"), pSizer);
 
-	ParseDN_by_prefix(parent, tokenlist, _T("CN"), _("Common name:"), pSizer);
-	ParseDN_by_prefix(parent, tokenlist, _T("O"), _("Organization:"), pSizer);
-	ParseDN_by_prefix(parent, tokenlist, _T("2.5.4.15"), _("Business category:"), pSizer, true);
-	ParseDN_by_prefix(parent, tokenlist, _T("OU"), _("Unit:"), pSizer);
-	ParseDN_by_prefix(parent, tokenlist, _T("T"), _("Title:"), pSizer);
-	ParseDN_by_prefix(parent, tokenlist, _T("C"), _("Country:"), pSizer);
-	ParseDN_by_prefix(parent, tokenlist, _T("ST"), _("State or province:"), pSizer);
-	ParseDN_by_prefix(parent, tokenlist, _T("L"), _("Locality:"), pSizer);
-	ParseDN_by_prefix(parent, tokenlist, _T("2.5.4.17"), _("Postal code:"), pSizer, true);
-	ParseDN_by_prefix(parent, tokenlist, _T("postalCode"), _("Postal code:"), pSizer, true);
-	ParseDN_by_prefix(parent, tokenlist, _T("STREET"), _("Street:"), pSizer);
-	ParseDN_by_prefix(parent, tokenlist, _T("EMAIL"), _("E-Mail:"), pSizer);
-	ParseDN_by_prefix(parent, tokenlist, _T("serialNumber"), _("Serial number:"), pSizer);
-	ParseDN_by_prefix(parent, tokenlist, _T("1.3.6.1.4.1.311.60.2.1.3"), _("Jurisdiction country:"), pSizer, true);
-	ParseDN_by_prefix(parent, tokenlist, _T("1.3.6.1.4.1.311.60.2.1.2"), _("Jurisdiction state or province:"), pSizer, true);
-	ParseDN_by_prefix(parent, tokenlist, _T("1.3.6.1.4.1.311.60.2.1.1"), _("Jurisdiction locality:"), pSizer, true);
-
-	if (!tokenlist.empty()) {
-		wxString value = tokenlist.front();
-		for (auto iter = ++tokenlist.cbegin(); iter != tokenlist.cend(); ++iter) {
-			value += _T(",") + *iter;
+	if (!tokens.empty()) {
+		std::wstring other;
+		for (auto const& pair : tokens) {
+			if (!other.empty()) {
+				other += ',';
+			}
+			other += pair.first;
+			other += '=';
+			other += pair.second;
 		}
 
 		pSizer->Add(new wxStaticText(parent, wxID_ANY, _("Other:")));
-		pSizer->Add(new wxStaticText(parent, wxID_ANY, value));
+		pSizer->Add(new wxStaticText(parent, wxID_ANY, LabelEscape(other)));
 	}
 }
 
-void CVerifyCertDialog::ParseDN_by_prefix(wxWindow* parent, std::list<wxString>& tokens, wxString prefix, const wxString& name, wxSizer* pSizer, bool decode)
+void CVerifyCertDialog::ParseDN_by_prefix(wxWindow* parent, std::vector<std::pair<std::wstring, std::wstring>> & tokens, std::wstring const& prefix, wxString const& name, wxSizer* pSizer)
 {
-	prefix += _T("=");
-	int len = prefix.Length();
+	std::wstring value;
 
-	wxString value;
-
-	bool append = false;
-
-	auto iter = tokens.begin();
-	while (iter != tokens.end()) {
-		if (!append) {
-			if (iter->Left(len) != prefix) {
-				++iter;
-				continue;
-			}
-
-			if (!value.empty()) {
-				value += _T("\n");
-			}
-		}
-		else {
-			append = false;
-			value += _T(",");
+	for (auto it = tokens.cbegin(); it != tokens.cend(); ) {
+		auto& pair = *it;
+		if (!fz::equal_insensitive_ascii(pair.first, prefix)) {
+			++it;
+			continue;
 		}
 
-		value += iter->Mid(len);
-
-		if (iter->Last() == '\\') {
-			value.RemoveLast();
-			append = true;
-			len = 0;
+		if (!value.empty()) {
+			value += '\n';
 		}
+		value += pair.second;
 
-		auto remove = iter++;
-		tokens.erase(remove);
-	}
-
-	if (decode) {
-		value = DecodeValue(value);
+		it = tokens.erase(it);
 	}
 
 	if (!value.empty()) {
 		pSizer->Add(new wxStaticText(parent, wxID_ANY, name));
-		pSizer->Add(new wxStaticText(parent, wxID_ANY, value));
+		pSizer->Add(new wxStaticText(parent, wxID_ANY, LabelEscape(value)));
 	}
 }
 
-wxString CVerifyCertDialog::DecodeValue(const wxString& value)
-{
-	// Decodes string in hex notation
-	// #xxxx466F6F626172 -> Foobar
-	// First two encoded bytes are ignored, some weird type information I don't care about
-	// Only accepts ASCII for now.
-	if (value.empty() || value[0] != '#') {
-		return value;
-	}
-
-	unsigned int len = value.Len();
-
-	wxString out;
-
-	for (unsigned int i = 5; i + 1 < len; i += 2) {
-		wxChar c = value[i];
-		wxChar d = value[i + 1];
-		if (c >= '0' && c <= '9') {
-			c -= '0';
-		}
-		else if (c >= 'a' && c <= 'z') {
-			c -= 'a' - 10;
-		}
-		else if (c >= 'A' && c <= 'Z') {
-			c -= 'A' - 10;
-		}
-		else {
-			continue;
-		}
-		if (d >= '0' && d <= '9') {
-			d -= '0';
-		}
-		else if (d >= 'a' && d <= 'z') {
-			d -= 'a' - 10;
-		}
-		else if (d >= 'A' && d <= 'Z') {
-			d -= 'A' - 10;
-		}
-		else {
-			continue;
-		}
-
-		c = c * 16 + d;
-		if (c > 127 || c < 32) {
-			continue;
-		}
-		out += c;
-	}
-
-	return out;
-}
-
 void CVerifyCertDialog::OnCertificateChoice(wxCommandEvent& event)
 {
 	int sel = event.GetSelection();
@@ -747,19 +723,19 @@ void ConfirmInsecureConection(CertStore & certStore, CInsecureFTPNotification &
 	bool const warning = certStore.HasCertificate(notification.server_.GetHost(), notification.server_.GetPort());
 
 	if (warning) {
-		main->Add(new wxStaticText(&dlg, -1, _T("Warning! You have previously connected to this server using FTP over TLS, yet the server has now rejected FTP over TLS.")));
-		main->Add(new wxStaticText(&dlg, -1, _T("This may be the result of a downgrade attack, only continue after you have spoken to the server administrator or server hosting provider.")));
+		main->Add(new wxStaticText(&dlg, -1, _("Warning! You have previously connected to this server using FTP over TLS, yet the server has now rejected FTP over TLS.")));
+		main->Add(new wxStaticText(&dlg, -1, _("This may be the result of a downgrade attack, only continue after you have spoken to the server administrator or server hosting provider.")));
 	}
 	else {
-		main->Add(new wxStaticText(&dlg, -1, _T("This server does not support FTP over TLS.")));
+		main->Add(new wxStaticText(&dlg, -1, _("This server does not support FTP over TLS.")));
 	}
-	main->Add(new wxStaticText(&dlg, -1, _T("If you continue, your password and files will be sent in clear over the internet.")));
+	main->Add(new wxStaticText(&dlg, -1, _("If you continue, your password and files will be sent in clear over the internet.")));
 
 
 	auto flex = lay.createFlex(2);
 	main->Add(flex, 0, wxALL, lay.border);
 	flex->Add(new wxStaticText(&dlg, -1, _("Host:")), lay.valign);
-	flex->Add(new wxStaticText(&dlg, -1, notification.server_.GetHost()), lay.valign);
+	flex->Add(new wxStaticText(&dlg, -1, LabelEscape(notification.server_.GetHost())), lay.valign);
 	flex->Add(new wxStaticText(&dlg, -1, _("Port:")), lay.valign);
 	flex->Add(new wxStaticText(&dlg, -1, fz::to_wstring(notification.server_.GetPort())), lay.valign);
 
